from typing import List, Optional

from aim.ext.resource import DEFAULT_SYSTEM_TRACKING_INT
from pydantic import BaseModel, Extra, Field


class UiOptions(BaseModel):
    """Options for the ui command."""

    class Config:
        extra = Extra.forbid

    port: int = Field(default=43800, description="Port to run the aim UI on.")
    host: str = Field(default="127.0.0.1", description="Host to run the aim UI on.")


class RunOptions(BaseModel):
    """Options for the run command."""

    class Config:
        extra = Extra.forbid

    run_hash: Optional[str] = Field(
        default=None,
        description=(
            "The hash of the run. If a run hash is selected that already "
            "exists, it will be logged to that run."
        ),
    )
    experiment: Optional[str] = Field(
        default=None,
        description=(
            "The name of the experiment. 'default’ if not specified. "
            "Can be used later to query runs/sequences"
        ),
    )
    system_tracking_interval: Optional[int] = Field(
        default=DEFAULT_SYSTEM_TRACKING_INT,
        description=(
            "Sets the tracking interval in seconds for system usage metrics "
            "(CPU, Memory, etc.). Set to None to disable system metrics tracking."
        ),
    )
    log_system_params: Optional[bool] = Field(
        default=False,
        description=(
            "Enable/Disable logging of system params such as installed packages, "
            "git info, environment variables, etc."
        ),
    )
    capture_terminal_logs: Optional[bool] = Field(
        default=True, description="Enable/Disable the capturing of terminal logs."
    )
    tags: List[str] = Field(
        default_factory=list, description="List of tags for the run."
    )


class RepositoryOptions(BaseModel):
    """Options for the repository."""

    class Config:
        extra = Extra.forbid

    path: Optional[str] = Field(
        default=None, description="Path to the repository folder."
    )
    read_only: Optional[bool] = Field(
        default=None, description="Enable/Disable writes to repository."
    )
    init: bool = Field(
        default=False,
        description="Enable/Disable initialilzation of repository folder before run.",
    )


class DisableOptions(BaseModel):
    """Options for the disable command."""

    pipelines: List[str] = Field(
        default_factory=list,
        description="List of pipelines in which tracking with aim will be disabled.",
    )


class KedroAimConfig(BaseModel):
    """The pydantic model for the `aim.yml` file which configures this plugin."""

    class Config:
        extra = Extra.forbid

    ui: UiOptions = Field(UiOptions(), description="Options for the aim ui.")
    run: RunOptions = Field(RunOptions(), description="Options for the aim run.")
    repository: RepositoryOptions = Field(
        RepositoryOptions(), description="Configurations for the aim repository."
    )
    disable: DisableOptions = Field(
        DisableOptions(), description="Options for disabling aim tracking."
    )



from enum import Enum
from logging import getLogger
from typing import Any, Dict, Optional

from aim import Run
from kedro.config import MissingConfigException
from kedro.framework.context import KedroContext
from kedro.framework.hooks import hook_impl
from kedro.io import DataCatalog, MemoryDataSet
from kedro.pipeline import Pipeline
from kedro.pipeline.node import Node

from kedro_aim.config import KedroAimConfig
from kedro_aim.config.utils import load_repository
from kedro_aim.framework.hooks.utils import check_aim_enabled
from kedro_aim.io.artifacts import AimArtifactDataSet, make_run_dataset

LOGGER = getLogger(__name__)


class StatusTag(str, Enum):
    """Enumeration of the status tags that can be used to tag metrics."""

    FAILURE = "failure"
    SUCCESS = "success"


class AimHook:
    """The hook that is used to integrate Aim with Kedro.

    The hook is responsible for:

    - Creating the Aim run before the pipeline is run.
    - Adding the Aim run to the catlog.
    """

    run: Optional[Run] = None
    aim_confg: KedroAimConfig

    @hook_impl
    def after_context_created(
        self,
        context: KedroContext,
    ) -> None:
        """Hooks to be invoked after a `KedroContext` is created.

        This hook reads the Aim configuration from the `aim.yml` from the `conf` folder
        of the Kedro project and stores it in the `aim_config` attribute of the hook.

        Args:
            context: The newly created context.
        """
        # Find the AimConfig in the context
        try:
            conf_aim_yml = context.config_loader.get("aim*", "aim*/**")
        except MissingConfigException:
            LOGGER.warning("No 'aim.yml' config file found in environment")
            conf_aim_yml = {}
        aim_config = KedroAimConfig.parse_obj(conf_aim_yml)

        # store in context for interactive use
        context.__setattr__("aim", aim_config)

        # store for further reuse
        self.aim_config = aim_config

    @hook_impl
    def after_catalog_created(
        self,
        catalog: DataCatalog,
        conf_catalog: Dict[str, Any],
        conf_creds: Dict[str, Any],
        feed_dict: Dict[str, Any],
        save_version: str,
        load_versions: str,
    ) -> None:
        """Hooks to be invoked after a data catalog is created.

        Im this hook we go through all the datasets in the catalog an replace the
        datasets that are of type `AimArtifactDataSet` with a special aim dataset.

        Args:
            catalog: The catalog that was created.
            conf_catalog: The config from which the catalog was created.
            conf_creds: The credentials conf from which the catalog was created.
            feed_dict: The feed_dict that was added to the catalog after creation.
            save_version: The save_version used in `save` operations
                for all datasets in the catalog.
            load_versions: The load_versions used in `load` operations
                for each dataset in the catalog.
        """
        # HACK: Replace all AimArtifactDataSet with a AimArtifactDataSetChild dataset.
        # This is needed to pass a reference of the run to the dataset.
        for name, dataset in catalog._data_sets.items():
            if isinstance(dataset, AimArtifactDataSet):
                catalog._data_sets[name] = make_run_dataset(self, dataset)

    @hook_impl
    def before_pipeline_run(
        self, run_params: Dict[str, Any], pipeline: Pipeline, catalog: DataCatalog
    ) -> None:
        """Hook to be invoked before a pipeline runs.

        Before the pipeline runs, we create the Aim run and add it to the catalog
        under the name `run`. This allows us to access the run in the pipeline.

        Args:
            run_params: The params used to run the pipeline.
                Should have the following schema

                ```json
                   {
                     "session_id": str,
                     "project_path": str,
                     "env": str,
                     "kedro_version": str,
                     "tags": Optional[List[str]],
                     "from_nodes": Optional[List[str]],
                     "to_nodes": Optional[List[str]],
                     "node_names": Optional[List[str]],
                     "from_inputs": Optional[List[str]],
                     "to_outputs": Optional[List[str]],
                     "load_versions": Optional[List[str]],
                     "pipeline_name": str,
                     "extra_params": Optional[Dict[str, Any]]
                   }
                ```

            pipeline: The `Pipeline` that will be run.
            catalog: The `DataCatalog` to be used during the run.
        """
        if check_aim_enabled(run_params["pipeline_name"], self.aim_config):
            # Create the Aim Run
            self.run = Run(
                run_hash=self.aim_config.run.run_hash,
                repo=load_repository(self.aim_config.repository),
                experiment=self.aim_config.run.experiment,
                system_tracking_interval=self.aim_config.run.system_tracking_interval,
                log_system_params=self.aim_config.run.log_system_params,
                capture_terminal_logs=self.aim_config.run.capture_terminal_logs,
            )

            # log run paramerters
            self.run["kedro"] = run_params

            # add tags
            for tag in self.aim_config.run.tags:
                self.run.add_tag(tag)

            # save run in catalog
            assert not catalog.exists("run"), "catalog already contains a 'run' dataset"
            catalog.add("run", MemoryDataSet(copy_mode="assign"))
            catalog.save("run", self.run)

    @hook_impl
    def before_node_run(
        self,
        node: Node,
        catalog: DataCatalog,
        inputs: Dict[str, Any],
        is_async: bool,
        session_id: str,
    ) -> None:
        """Hook to be invoked before a node runs.

        All `parameters` that are passed to the node are logged to the run.

        Args:
            node: The `Node` to run.
            catalog: A `DataCatalog` containing the node's inputs and outputs.
            inputs: The dictionary of inputs dataset.
                The keys are dataset names and the values are the actual loaded input
                data, not the dataset instance.
            is_async: Whether the node was run in `async` mode.
            session_id: The id of the session.
        """
        if self.run is not None:
            # only parameters will be logged.
            for k, v in inputs.items():
                if k.startswith("params:"):
                    self.run[k[7:]] = v
                elif k == "parameters":
                    self.run[k] = v

    @hook_impl
    def after_pipeline_run(
        self,
        run_params: Dict[str, Any],
        pipeline: Pipeline,
        catalog: DataCatalog,
    ) -> None:
        """Hook to be invoked after a pipeline runs.

        After the pipeline runs, we close the Aim run and add `StatusTag.SUCCESS` tag.

        Args:
            run_params: The params used to run the pipeline.
                Should have the following schema

                ```json
                   {
                     "session_id": str,
                     "project_path": str,
                     "env": str,
                     "kedro_version": str,
                     "tags": Optional[List[str]],
                     "from_nodes": Optional[List[str]],
                     "to_nodes": Optional[List[str]],
                     "node_names": Optional[List[str]],
                     "from_inputs": Optional[List[str]],
                     "to_outputs": Optional[List[str]],
                     "load_versions": Optional[List[str]],
                     "pipeline_name": str,
                     "extra_params": Optional[Dict[str, Any]]
                   }
                ```

            pipeline: The `Pipeline` that was run.
            catalog: The `DataCatalog` used during the run.
        """
        if self.run is not None:
            self.run.add_tag(StatusTag.SUCCESS)
            self.run.finalize()
            self.run.close()

    @hook_impl
    def on_pipeline_error(
        self,
        error: Exception,
        run_params: Dict[str, Any],
        pipeline: Pipeline,
        catalog: DataCatalog,
    ) -> None:
        """Hook to be invoked if a pipeline run throws an uncaught Exception.

        In this case, we close the run and add a `StatusTag.FAILURE` tag.

        Args:
            error: The uncaught exception thrown during the pipeline run.
            run_params: The params used to run the pipeline.
                Should have the following schema

                ```json
                   {
                     "session_id": str,
                     "project_path": str,
                     "env": str,
                     "kedro_version": str,
                     "tags": Optional[List[str]],
                     "from_nodes": Optional[List[str]],
                     "to_nodes": Optional[List[str]],
                     "node_names": Optional[List[str]],
                     "from_inputs": Optional[List[str]],
                     "to_outputs": Optional[List[str]],
                     "load_versions": Optional[List[str]],
                     "pipeline_name": str,
                     "extra_params": Optional[Dict[str, Any]]
                   }
                ```
            pipeline: The ``Pipeline`` that will was run.
            catalog: The ``DataCatalog`` used during the run.
        """
        if self.run is not None:
            self.run.add_tag(StatusTag.FAILURE)
            self.run.finalize()
            self.run.close()


aim_hook = AimHook()



from kedro_aim.config import KedroAimConfig


def check_aim_enabled(pipeline_name: str, aim_config: KedroAimConfig) -> bool:
    """Check if Aim is enabled for the given pipeline.

    Args:
        pipeline_name: Name of the pipeline.
        aim_config: Kedro-Aim configuration.

    Returns:
        A boolean indicating whether Aim is enabled for the given pipeline.
    """
    return pipeline_name not in aim_config.disable.pipelines



from enum import Enum
from logging import getLogger
from typing import Any, Dict, Optional

from aim import Audio, Figure, Image, Text
from kedro.io import AbstractDataSet
from kedro.io.core import parse_dataset_definition

from kedro_aim.aim.utils import list_metrics_names_in_run
from kedro_aim.framework import hooks

LOGGER = getLogger(__name__)


class ArtifactType(str, Enum):
    """Enumeration of the artifact types that can be used to tag metrics."""

    IMAGE = "image"
    FIGURE = "figure"
    TEXT = "text"
    AUDIO = "audio"


class AimArtifactDataSet(AbstractDataSet[Any, Any]):
    """A dataset that is used to save artifacts to Aim.

    This dataset does not implement any functionality itself. Instead it is used to
    indicate to the `AimHook` that the artifact should be saved to Aim.
    During the `after_catalog_created` hook, the `AimHook` will replace the dataset
    with the actual dataset that is used to save the artifact to Aim.

    Args:
        artifact_type: The type of the artifact.
        name: The name that is used to save the artifact to Aim.
        data_set: The dataset that is used to load the artifact.
        context: The run context of artifact. Defaults to {}.
        save_args: Additional parameters that are passed to aim. Defaults to None.
    """

    def __init__(
        self,
        artifact_type: ArtifactType,
        name: str,
        data_set: Dict[str, Any],
        context: Dict[str, Any] = {},
        save_args: Optional[Dict[str, Any]] = None,
    ) -> None:
        assert (
            artifact_type in ArtifactType.__members__.values()
        ), f"Invalid artifact type `{artifact_type}`."

        self.artifact_type = artifact_type
        self.data_set = data_set
        self.name = name
        self.context = context
        self.save_args = save_args

    def _load(self) -> Any:  # pragma: no cover
        raise NotImplementedError(
            "This method should be overwritten by `AimArtifactDataSetChild`."
        )

    def _save(self, data: Any) -> None:  # pragma: no cover
        raise NotImplementedError(
            "This method should be overwritten by `AimArtifactDataSetChild`."
        )

    def _describe(self) -> Dict[str, Any]:  # pragma: no cover
        raise NotImplementedError(
            "This method should be overwritten by `AimArtifactDataSetChild`."
        )


def make_run_dataset(
    hook: "hooks.AimHook",
    artifact_dataset: AimArtifactDataSet,
) -> AbstractDataSet[Any, Any]:
    """Takes `AimArtifactDataSet` and replaces it with `AimArtifactDataSetChild`.

    Replacing the dataset is necessary because the `AimArtifactDataSetChild` is the
    actual dataset that is used to save the artifact to Aim. The `AimArtifactDataSet`
    is only used as a placeholder to indicate to the `AimHook` that the artifact should
    be saved to Aim.  This is necessary because we need to wait until the
    `after_catalog_created` hook to pass a reference to the `AimHook` to the dataset.

    Args:
        hook: The `AimHook` hook
        artifact_dataset: The placeholder dataset

    Returns:
        A dataset that is used to save the artifact to Aim.
    """
    data_set_cls, data_set_args = parse_dataset_definition(
        config=artifact_dataset.data_set
    )
    save_args = {} if artifact_dataset.save_args is None else artifact_dataset.save_args

    class AimArtifactDataSetChild(data_set_cls):  # type: ignore
        def __init__(self, *args: Any, **kwargs: Any) -> None:
            super().__init__(*args, **kwargs)

        def _save(self, data: Any) -> None:
            self._track_artifact(data, artifact_dataset.artifact_type)
            super()._save(data)

        def _load(self) -> Any:
            data = super()._load()

            # track artifact if it was not tracked before
            if hook.run is not None:
                if artifact_dataset.name not in list_metrics_names_in_run(hook.run):
                    self._track_artifact(data, artifact_dataset.artifact_type)

            return data

        def _describe(self) -> Dict[str, Any]:
            return super()._describe()

        @staticmethod
        def _track_artifact(data: Any, artifact_type: ArtifactType) -> None:
            if hook.run is not None:
                if artifact_type == ArtifactType.IMAGE:
                    tracked_data = Image(data, **save_args)
                elif artifact_type == ArtifactType.FIGURE:
                    tracked_data = Figure(data, **save_args)
                elif artifact_type == ArtifactType.TEXT:
                    tracked_data = Text(data, **save_args)
                elif artifact_type == ArtifactType.AUDIO:
                    tracked_data = Audio(data, **save_args)
                else:
                    raise AssertionError(f"Invalid artifact type `{artifact_type}`.")

                hook.run.track(
                    value=tracked_data,
                    name=artifact_dataset.name,
                    context=artifact_dataset.context,  # type: ignore
                )
            else:
                LOGGER.warning("No run is active. Skipping artifact tracking.")

    # rename the class
    parent_name = data_set_cls.__name__
    AimArtifactDataSetChild.__name__ = f"Mlflow{parent_name}"
    AimArtifactDataSetChild.__qualname__ = f"{parent_name}.Mlflow{parent_name}"

    aim_dataset_instance = AimArtifactDataSetChild(**data_set_args)
    return aim_dataset_instance



from typing import Generator

from aim import Run
from aim.sdk.query_utils import SequenceView
from aim.sdk.sequence import Sequence


def list_metrics_in_run(run: Run) -> Generator[SequenceView, None, None]:
    """List all metrics in the run.

    HACK: This is a workaround for the `metrics` property of `aim.Run` which is broken.
    In the future, we should use the `metrics` property of `aim.Run` instead of this.

    Args:
        run: A aim run object.

    Yields:
        The metrics that are contained in the run.
    """
    for seq_name, ctx, run in run.iter_sequence_info_by_type("*"):
        yield Sequence(seq_name, ctx, run)  # type: ignore


def list_metrics_names_in_run(run: Run) -> Generator[str, None, None]:
    """List the names of all metrics in the run.

    Args:
        run: A aim run object.

    Yields:
        The names of the metrics that are contained in the run.
    """
    for metric in list_metrics_in_run(run):
        yield metric.name



from typing import Optional

from aim.sdk.repo import Repo

from kedro_aim.config.model import RepositoryOptions


def load_repository(cfg: RepositoryOptions) -> Optional[Repo]:
    """Load the a aim repository from the config options.

    Args:
        cfg: Config for the repository.

    Returns:
        A aim repository or None if the repository is disabled.
    """
    if cfg.path is None:
        return None
    else:
        return Repo(path=cfg.path, read_only=cfg.read_only, init=cfg.init)



from pathlib import Path
from typing import Any, Union

from jinja2 import Environment, FileSystemLoader


def render_jinja_template(
    src: Union[str, Path], is_cookiecutter: bool = False, **kwargs: Any
) -> str:  # pragma: no cover
    """Render a template file and replace tis jinja's tags.

    This functions enable to copy a file and render the tags (identified by
    `{{ my_tag }}`) with the values provided in kwargs.

    Args:
        src:  The path to the template which should be rendered
        is_cookiecutter: If the template is a cookiecutter template
        **kwargs: Extra arguments to be passed to the template

    Returns:
        A string that contains all the files with replaced tags.
    """
    src = Path(src)

    template_loader = FileSystemLoader(searchpath=src.parent.as_posix())
    # the keep_trailing_new_line option is mandatory to
    # make sure that black formatting will be preserved
    template_env = Environment(loader=template_loader, keep_trailing_newline=True)
    template = template_env.get_template(src.name)
    if is_cookiecutter:
        # we need to match tags from a cookiecutter object
        # but cookiecutter only deals with folder, not file
        # thus we need to create an object with all necessary attributes
        class FalseCookieCutter:
            def __init__(self, **kwargs: Any) -> None:
                self.__dict__.update(kwargs)

        parsed_template = template.render(cookiecutter=FalseCookieCutter(**kwargs))
    else:
        parsed_template = template.render(**kwargs)

    return parsed_template


def write_jinja_template(
    src: Union[str, Path], dst: Union[str, Path], **kwargs: Any
) -> None:
    """Write a template file and replace tis jinja's tags.

    Write a template file and replace tis jinja's tags (identified by `{{ my_tag }}`)
    with the values provided in kwargs.

    Args:
        src: Path to the template which should be rendered
        dst: Path where the rendered template should be saved
        **kwargs: Extra arguments to be passed to the template
    """
    dst = Path(dst)
    parsed_template = render_jinja_template(src, **kwargs)
    with open(dst, "w") as file_handler:
        file_handler.write(parsed_template)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['kedro_aim',
 'kedro_aim.aim',
 'kedro_aim.config',
 'kedro_aim.framework',
 'kedro_aim.framework.cli',
 'kedro_aim.framework.hooks',
 'kedro_aim.io',
 'kedro_aim.io.artifacts',
 'kedro_aim.template']

package_data = \
{'': ['*'], 'kedro_aim.template': ['config/*']}

install_requires = \
['aim>=3.14.1,<4.0.0', 'kedro>=0.18.0', 'pydantic>=1.10.2,<2.0.0']

entry_points = \
{'kedro.hooks': ['aim_hook = kedro_aim.framework.hooks.aim_hook:aim_hook'],
 'kedro.project_commands': ['kedro_aim = kedro_aim.framework.cli.cli:commands']}

setup_kwargs = {
    'name': 'kedro-aim',
    'version': '0.1.3',
    'description': 'A plugin to integrate the mlops plattform aim into your kedro project',
    'long_description': '# Kedro Aim\n\n[![PyPI version](https://badge.fury.io/py/kedro-aim.svg)](https://badge.fury.io/py/kedro-aim)\n[![Python version](https://img.shields.io/badge/python-3.8|3.9|3.10-blue.svg)](https://pypi.org/project/kedro/)\n[![Documentation Status](https://readthedocs.org/projects/kedro-aim/badge/?version=latest)](https://kedro-aim.readthedocs.io/en/latest/?badge=latest)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Publish Package to PyPI](https://github.com/AnH0ang/kedro-aim/actions/workflows/publish.yml/badge.svg)](https://github.com/AnH0ang/kedro-aim/actions/workflows/publish.yml)\n[![Testing](https://github.com/AnH0ang/kedro-aim/actions/workflows/testing.yml/badge.svg)](https://github.com/AnH0ang/kedro-aim/actions/workflows/testing.yml)\n[![codecov](https://codecov.io/gh/AnH0ang/kedro-aim/branch/master/graph/badge.svg?token=X94NV660A9)](https://codecov.io/gh/AnH0ang/kedro-aim)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n## 📝 Description\n\n`kedro-aim` is a [kedro-plugin](https://kedro.readthedocs.io/en/stable/extend_kedro/plugins.html) that enables tracking of metrics and parameters with [Aim](https://aimstack.io/) from within Kedro.\nKedro is a great tool for data engineering and data science, but it lacks a clear way to track metrics and parameters.\nAim is a great tool for tracking metrics and parameters, but it lacks a clear way to integrate with Kedro.\nThis plugin aims to solve both problems.\n\n![Aim Screenshot](./static/img/aim-screenshot.png)\n\n## 🎖 Features\n\n- Automatic Registration of Aim `Run` in Data Catalog\n- Tracking of Artifact with Aim DataSet\n- Configuration over `aim.yml`\n\n## ⚙️ Installation\n\nInstall the package with `pip`:\n\n```bash\npip install kedro-aim\n```\n\n## 💡 Usage Examples\n\nThe plugin automatically registers a [Run](https://aimstack.readthedocs.io/en/latest/refs/sdk.html#aim.sdk.run.Run) instance in the DataCatalog under the name `run` which can be accessed by all nodes to log metrics and parameters.\nThis run instance can be used track metrics and parameters in the same way as in any other [python project](https://aimstack.readthedocs.io/en/latest/quick_start/supported_types.html)\n\nFirst you need to initilize the `aim.yml` config file inside your pre-existing Kedro project.\nThis can be done by running the following command:\n\n```shell\nkedro aim init\n```\n\nIn order to use `aim` inside a node you need to pass the run object as a argument of the function.\nInside the function you can access the run object and use it to log metrics and parameters.\n\n```python\n# nodes.py\nimport pandas as pd\nfrom aim import Run\n\n\ndef logging_in_node(run: Run, data: pd.DataFrame) -> None:\n    # track metric\n    run.track(0.5, "score")\n\n    # track parameter\n    run["parameter"] = "abc"\n```\n\nWhen defining the pipeline, you need to pass the `run` dataset as a input to the node.\nThe `run` dataset will be automatically created by `kedro-aim` and added to the DataCatalog.\nAs a result, the `run` dataset will be passed to the node as an argument.\n\n```python\n# pipeline.py\nfrom kedro.pipeline import node, Pipeline\nfrom kedro.pipeline.modular_pipeline import pipeline\n\n\ndef create_pipeline(**kwargs) -> Pipeline:\n    return pipeline(\n        [\n            node(\n                func=logging_in_node,\n                inputs=["run", "input_data"],\n                outputs=None,\n                name="logging_in_node",\n            )\n        ]\n    )\n```\n\n## 🧰 Config File\n\nThe module is configured via the `aim.yml` file which should be placed inside the `conf/base` folder.\nA default config file can be generated using the `kedro aim init` command from the shell.\n\nYou can enable the schema validation in your VSCode IDE to enable real-time validation, autocompletion and see information about the different fields in your catalog as you write it. To enable this, make sure you have the [YAML plugin](https://marketplace.visualstudio.com/items?itemName=redhat.vscode-yaml) installed.\nThen enter the following in your `settings.json` file:\n\n```json\n{\n  "yaml.schemas": {\n    "https://raw.githubusercontent.com/AnH0ang/kedro-aim/master/static/jsonschema/kedro_aim_schema.json": "**/*aim*.yml"\n  }\n}\n```\n\n## 🙏 Acknowledgement\n\nThis project was inspired by the work of [kedro-mlflow](https://github.com/Galileo-Galilei/kedro-mlflow) which is a plugin for Kedro that enables tracking of metrics and parameters with [MLflow](https://mlflow.org/) from within Kedro.\n',
    'author': 'An Hoang',
    'author_email': 'anhoang31415@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://kedro-aim.readthedocs.io',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



import subprocess
from logging import getLogger
from pathlib import Path
from typing import Dict, List, Optional

import click
from click.core import Command, Context
from kedro.framework.project import settings
from kedro.framework.session import KedroSession
from kedro.framework.startup import _is_project, bootstrap_project

from kedro_aim.config import KedroAimConfig
from kedro_aim.framework.cli.cli_utils import write_jinja_template

LOGGER = getLogger(__name__)
TEMPLATE_FOLDER_PATH = Path(__file__).parent.parent.parent / "template" / "config"


class KedroClickGroup(click.Group):
    """The main entry point for the Kedro CLI."""

    def reset_commands(self) -> None:
        """Reset the commands to the default ones."""
        self.commands: Dict[str, Command] = {}

        # add commands on the fly based on conditions
        if _is_project(Path.cwd()):
            self.add_command(init)  # type: ignore
            self.add_command(ui)  # type: ignore

    def list_commands(self, ctx: Context) -> List[str]:
        """List the names of all commands.

        Args:
            ctx: The click context.

        Returns:
            A list of command names.
        """
        self.reset_commands()
        commands_list = sorted(self.commands)
        return commands_list

    def get_command(self, ctx: Context, cmd_name: str) -> Optional[click.Command]:
        """Get a click command by name.

        Args:
            ctx: The click context.
            cmd_name: The name of the command.

        Returns:
            The click command with the given name.
        """
        self.reset_commands()
        return self.commands.get(cmd_name)


@click.group(name="Aim")
def commands() -> None:
    """Kedro plugin for interactions with aim."""
    pass  # pragma: no cover


@commands.command(name="aim", cls=KedroClickGroup)
def aim_commands() -> None:
    """Use aim-specific commands inside kedro project."""
    pass  # pragma: no cover


@aim_commands.command()  # type: ignore
@click.option(
    "--env",
    "-e",
    default="local",
    help=(
        "The name of the kedro environment where the 'aim.yml' should be created. "
        "Default to 'local'"
    ),
)
@click.option(
    "--force",
    "-f",
    is_flag=True,
    default=False,
    help="Update the template without any checks.",
)
@click.option(
    "--silent",
    "-s",
    is_flag=True,
    default=False,
    help="Should message be logged when files are modified?",
)
def init(env: str, force: bool, silent: bool) -> None:
    """Updates the template of a kedro project.

    Running this command is mandatory to use kedro-aim.
    This adds "conf/base/aim.yml": This is a configuration file
    used for run parametrization when calling "kedro run" command.
    """
    # get constants
    aim_yml = "aim.yml"
    project_path = Path().cwd()
    project_metadata = bootstrap_project(project_path)
    aim_yml_path: Path = (
        project_path / settings.CONF_SOURCE / env / aim_yml  # type: ignore
    )

    # aim.yml is just a static file,
    # but the name of the experiment is set to be the same as the project
    if aim_yml_path.is_file() and not force:
        click.secho(
            click.style(
                (
                    f"A 'aim.yml' already exists at '{aim_yml_path}'. "
                    "You can use the ``--force`` option to override it."
                ),
                fg="red",
            )
        )
    else:
        try:
            write_jinja_template(
                src=TEMPLATE_FOLDER_PATH / aim_yml,
                is_cookiecutter=False,
                dst=aim_yml_path,
                python_package=project_metadata.package_name,
            )
            if not silent:
                click.secho(
                    click.style(
                        (
                            f"'{settings.CONF_SOURCE}/{env}/{aim_yml}' "
                            "successfully updated."
                        ),
                        fg="green",
                    )
                )
        except FileNotFoundError:
            click.secho(
                click.style(
                    (
                        f"No env '{env}' found. "
                        "Please check this folder exists inside "
                        f"'{settings.CONF_SOURCE}' folder.",
                    ),
                    fg="red",
                )
            )


@aim_commands.command()  # type: ignore
@click.option(
    "--env",
    "-e",
    required=False,
    default="local",
    help="The environment within conf folder we want to retrieve.",
)
@click.option(
    "--port",
    "-p",
    required=False,
    type=int,
    help="The port to listen on",
)
@click.option(
    "--host",
    "-h",
    required=False,
    help=(
        "The network address to listen on (default: 127.0.0.1). "
        "Use 0.0.0.0 to bind to all addresses if you want to access the tracking "
        "server from other machines."
    ),
)
def ui(env: str, port: int, host: str) -> None:
    """Start the aim UI.

    Opens the aim user interface with the project-specific settings of aim.yml.
    This interface enables to browse and compares runs.
    """
    project_path = Path().cwd()
    bootstrap_project(project_path)
    with KedroSession.create(
        project_path=project_path,
        env=env,
    ) as session:

        context = session.load_context()
        aim_config: KedroAimConfig = context.aim  # type: ignore
        host = host or aim_config.ui.host
        port = port or aim_config.ui.port

        subprocess.call(
            [
                "aim",
                "up",
                "--port",
                str(port),
                "--host",
                host,
            ]
        )
